% ----------------------------------------------------------------------- %
% main_type1_w_prescmot_share.m
%
% Author: Hideyuki Kimpara, Worcester Polytechnic Institute
%
% This code uses the Matlab interface to the OpenSim API to run a trial
% to connect simulink simulator and OpenSim analysis. 
% This example borrows from other Matlab and C++ examples avaiable from
% the OpenSim project written by several people including, but not limited
% to: Brian Umberger, Ajay Seth, Ayman Habib and Sam Hamner.
% ----------------------------------------------------------------------- %

clear

% Node and timing information
TimeData = struct();
TimeData.time_start = 0.0;
TimeData.time_end   = 2.0;

% Import the OpenSim modeling classes
import org.opensim.modeling.*

% Create an OpenSim model from an OSIM file
% Load the .osim file
Model_In = uigetfile({'*.osim', 'OpenSim Model Files (*.osim)'}, ...
                         'Select the OpenSim model file','MultiSelect', 'off');
% Model_In = '../Model/input_osimModel.osim'
fileoutpath = [Model_In(1:end-5),'_Prescribed.osim'];
osimModel = Model(Model_In);


% Add Analyses to the model: ForceReporter and BodyKinematics 
aForceReporter = ForceReporter(osimModel);
aActuation = Actuation(osimModel);
aKinematics = Kinematics(osimModel);
aBodyKinematics = BodyKinematics(osimModel);
aJointReaction = JointReaction(osimModel);
osimModel.addAnalysis(aForceReporter);
osimModel.addAnalysis(aActuation);
osimModel.addAnalysis(aKinematics);
osimModel.addAnalysis(aBodyKinematics);
osimModel.addAnalysis(aJointReaction);


% Initialize the model (build the system and intialize the state)
osimState = osimModel.initSystem();


% Load the .mot file
motfilepath = uigetfile({'*.mot', 'OpenSim Motion Files (*.mot)'}, ...
                         'Select the motion file','MultiSelect', 'off');
% motfilepath = 'Desired_motion.mot';
mot_func = importdata(motfilepath); % import motion data (1st column is time)
old_time_m = mot_func.data(:,1);         % time 
old_data_m = mot_func.data(:,2:end);     % the 6 states
old_text_m = mot_func.textdata(7,2:end); % states names


% get coordinate set from model, and count the number of coordinates
modelCoordSet = osimModel.getCoordinateSet();
nCoords = modelCoordSet.getSize();

% for all coordinates in the model, create a function and prescribe
for i=0:nCoords-1
    
    % Get the coordinate set from the model
	currentcoord = modelCoordSet.get(i);

    for j=1:size(old_data_m,2)
        if strcmp(currentcoord.getName(), old_text_m(j))

            % Check if it is a rotational or translational coordinate
            motion = currentcoord.getMotionType();

            % construct a SimmSpline object (previously NaturalCubicSpline)
            Spline = SimmSpline();

            %Now to write Time and coordvalue to Spline
            if strcmp(motion,'Rotational')
                % if the motion type is rotational we must convert to radians from degrees
                for j = 1:size(old_data_m,1)
                    Spline.addPoint(old_time_m(j),old_data_m(j)/(180/pi));
                end
            else % else we assume it's translational and can be left 'as is'
                for j = 1:size(old_data_m,1)
                    Spline.addPoint(old_time_m(j),old_data_m(j));
                end
            end

            % Add the SimmSpline to the PrescribedFunction of the Coordinate
            % being edited
            currentcoord.setPrescribedFunction(Spline);
            currentcoord.setDefaultIsPrescribed(1);
        end
    end

end


% % Initialize the model (build the system and intialize the state)
% osimState = osimModel.initSystem();

% Get the number of states, coordinates, muscles and controls from the model;
% in this case the number of controls equals the number of muscles
Nstates       = osimModel.getNumStateVariables();
Ncontrols     = osimModel.getNumControls();
Ncoord        = osimModel.getNumCoordinates(); 
model_muscles = osimModel.getMuscles();
Nmuscles      = model_muscles.getSize();
model_act     = osimModel.getActuators();
Nacts         = model_act.getSize();

states_all = cell(Nstates,1);
for i = 1:Nstates
   states_all(i,1) = cell(osimModel.getStateVariableNames().getitem(i-1));
end


% Load the .sto file that contain the initial guess for the states
init_file = uigetfile({'*.sto', 'OpenSim States Files (*.sto)'}, ...
                         'Select the initial states file','MultiSelect', 'off');
temp_s = importdata(init_file); % import states data
old_time_s = temp_s.data(:,1);         % time 
old_data_s = temp_s.data(:,2:end);     % the 6 states
old_text_s = temp_s.textdata(7,2:end); % states names

% Arrange the initial guess by nodes and states
x0_temp = zeros(Nstates,1);  % pre-allocate space
chk_counter =0;
for j = 1:size(states_all,1) 
    for k = 1:size(old_data_s,2)
        if strcmp(old_text_s(k),states_all(j)) == 1,
            % interpolate initial guess to the defined temporal grid
            x0_temp(j) = old_data_s(1,k);
            chk_counter = chk_counter+1;
        end
    end
    if chk_counter ~= j
        disp(['mis-matching' states_all(j)]);
        disp([states_all(j) char(j)]);
    end
end


%# Set the initial states of the model
editableCoordSet = osimModel.updCoordinateSet();
for j = 1:size(states_all,1)
    str_addr = strfind(char(states_all(j)),'_u');
    if (str_addr & length(char(states_all(j)))-str_addr(end)==1)
        temp_str=char(states_all(j));
        editableCoordSet.get(temp_str(1:str_addr-1)).setSpeedValue(osimState, x0_temp(j));        
    else
        editableCoordSet.get(states_all(j)).setValue(osimState, x0_temp(j));
    end
end


% Call the function that runs the simulation
osimModel.setPropertiesFromState(osimState);

osimModel.equilibrateMuscles(osimState);


%  Save the Modified Model to a file
osimModel.print(fileoutpath);
disp(['The new model has been saved at ' fileoutpath]);




% Create a manager to run the simulation
simulationManager = Manager(osimModel);
simulationManager.setWriteToStorage(true);
simulationManager.setPerformAnalyses(true);
simulationManager.setInitialTime(TimeData.time_start);
simulationManager.setFinalTime(TimeData.time_end);
simulationManager.setIntegratorAccuracy(1e-04)
simulationManager.integrate(osimState);


% Write out storage files containing the states and excitations
outputpath = '.\Result\';  % use current directory
bufferstamp = char(datetime,'yy_MM_dd-HH_mm');
simulationManager.getStateStorage().print(['SimOut_',bufferstamp,'_states.sto']);
osimModel.printControlStorage(['SimOut_',bufferstamp,'_excitations.sto']);

% Also write out the results of the analyses to storage files
aActuation.printResults(['SimOut_',bufferstamp],outputpath,0.01,'.sto');
aBodyKinematics.printResults(['SimOut_',bufferstamp],outputpath,0.01,'.sto');
aForceReporter.printResults(['SimOut_',bufferstamp],outputpath,0.01,'.sto');
aJointReaction.printResults(['SimOut_',bufferstamp],outputpath,0.01,'.sto');

% Reset the analyses so everything starts fresh for the next
% call to this function
aBodyKinematics.getPositionStorage.reset(0);
aBodyKinematics.getVelocityStorage.reset(0);
aBodyKinematics.getAccelerationStorage.reset(0);
aForceReporter.getForceStorage.reset(0);
